'use strict';

var _angular = require('angular');

var _angular2 = _interopRequireDefault(_angular);

var _tester = require('tester');

var _tester2 = _interopRequireDefault(_tester);

var _lodash = require('lodash.size');

var _lodash2 = _interopRequireDefault(_lodash);

var _ = require('../');

var _2 = _interopRequireDefault(_);

var _fixtures = require('./fixtures/');

var _fixtures2 = _interopRequireDefault(_fixtures);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

describe('avUserAuthorizations', function () {

  var tester = new _tester2.default();

  var avUserAuthorizations = void 0;

  var validatePermission = function validatePermission(permission) {
    expect(permission).toBeDefined();
    expect(permission.id).toBe('452');
    expect(permission.isAuthorized).toBe(true);
    expect(permission.organizations.length).toBe(1);
  };

  beforeEach(function () {
    return _angular2.default.mock.module(_2.default.name);
  });

  beforeEach(inject(function (_$httpBackend_, _avUserAuthorizations_, avUserPermissionsResource) {
    avUserAuthorizations = _avUserAuthorizations_;
    avUserPermissionsResource.sessionDate = null; // clear session date for tests
  }));

  tester.service();

  it('should exist', function () {
    expect(avUserAuthorizations).toBeDefined();
    expect(avUserAuthorizations.region).toBeNull();
  });

  describe('getPermissions()', function () {

    it('should return valid permissions', function () {

      tester.$httpBackend.expect('GET', _fixtures2.default.URI_452_999).respond(200, _fixtures2.default.VALID);
      avUserAuthorizations.getPermissions(['452', '999']).then(function (permissions) {
        validatePermission(permissions[0]);
      });
      tester.$httpBackend.flush();
    });

    it('should include ALL region in request', function () {

      avUserAuthorizations.setRegion('ALL');
      tester.$httpBackend.expect('GET', _fixtures2.default.URI_452_999_ALL).respond(200, _fixtures2.default.VALID);
      avUserAuthorizations.getPermissions(['452', '999']).then(function (permissions) {
        expect((0, _lodash2.default)(permissions)).toBe(2);
        validatePermission(permissions[0]);
      });
      tester.$httpBackend.flush();
    });

    it('should include merged permission ids', function () {

      tester.$httpBackend.expect('GET', _fixtures2.default.URI_452_999).respond(200, _fixtures2.default.VALID);
      avUserAuthorizations.getPermissions(['452', '999']);
      expect(avUserAuthorizations.permissionIds.length).toBe(2);
      tester.$httpBackend.flush();

      tester.$httpBackend.expect('GET', _fixtures2.default.URI_452_999_1000).respond(200, _fixtures2.default.VALID);
      avUserAuthorizations.getPermissions(['452', '1000']);
      expect(avUserAuthorizations.permissionIds.length).toBe(3);
      tester.$httpBackend.flush();
    });
  });

  describe('getPermission()', function () {

    it('should get single permission', function () {

      tester.$httpBackend.expect('GET', _fixtures2.default.URI_452).respond(200, _fixtures2.default.VALID);
      avUserAuthorizations.getPermission('452').then(function (permission) {
        validatePermission(permission);
      });
      tester.$httpBackend.flush();
    });

    it('should NOT have permission 999', function () {

      tester.$httpBackend.expect('GET', _fixtures2.default.URI_999).respond(200, _fixtures2.default.EMPTY);
      avUserAuthorizations.getPermission('999').then(function (permission) {
        expect(permission.isAuthorized).toBeFalsy();
      });
      tester.$httpBackend.flush();
    });
  }); // getPermission()

  describe('isAuthorized()', function () {

    it('should return true when authorized', function () {

      tester.$httpBackend.expect('GET', _fixtures2.default.URI_452).respond(200, _fixtures2.default.VALID);
      avUserAuthorizations.isAuthorized('452').then(function (isAuthorized) {
        expect(isAuthorized).toBe(true);
      });
      tester.$httpBackend.flush();
    });

    it('should return false when NOT authorized', function () {

      tester.$httpBackend.expect('GET', _fixtures2.default.URI_999).respond(200, _fixtures2.default.EMPTY);
      avUserAuthorizations.isAuthorized('999').then(function (isAuthorized) {
        expect(isAuthorized).toBe(false);
      });
      tester.$httpBackend.flush();
    });
  }); // isAuthorized()

  describe('isAnyAuthorized()', function () {

    it('should return true when permission is valid', function () {

      tester.$httpBackend.expect('GET', _fixtures2.default.URI_452).respond(200, _fixtures2.default.VALID);
      avUserAuthorizations.isAnyAuthorized(['452']).then(function (isAuthorized) {
        expect(isAuthorized).toBe(true);
      });
      tester.$httpBackend.flush();
    });

    it('should return false permission is invalid', function () {

      tester.$httpBackend.expect('GET', _fixtures2.default.URI_999).respond(200, _fixtures2.default.EMPTY);
      avUserAuthorizations.isAnyAuthorized(['999']).then(function (isAuthorized) {
        expect(isAuthorized).toBe(false);
      });
      tester.$httpBackend.flush();
    });

    it('should return true if either permission is valid', function () {

      tester.$httpBackend.expect('GET', _fixtures2.default.URI_452_999).respond(200, _fixtures2.default.VALID);
      avUserAuthorizations.isAnyAuthorized(['452', '999']).then(function (isAuthorized) {
        expect(isAuthorized).toBe(true);
      });
      tester.$httpBackend.flush();
    });
  });

  describe('getOrganizations()', function () {

    it('should have one organization for 452', function () {

      tester.$httpBackend.expect('GET', _fixtures2.default.URI_452).respond(200, _fixtures2.default.VALID);
      avUserAuthorizations.getOrganizations('452').then(function (organizations) {
        expect(Array.isArray(organizations)).toBe(true);
        expect(organizations.length).toBe(1);
        var org = organizations[0];
        expect(org.id).toBe('1435');
        expect(org.customerId).toBe('1001');
        expect(org.name).toBe('sample org name');
      });
      tester.$httpBackend.flush();
    });

    it('should no organizations for 999', function () {

      tester.$httpBackend.expect('GET', _fixtures2.default.URI_999).respond(200, _fixtures2.default.EMPTY);
      avUserAuthorizations.getOrganizations('999').then(function (organizations) {
        expect(Array.isArray(organizations)).toBe(true);
        expect(organizations.length).toBe(0);
      });
      tester.$httpBackend.flush();
    });
  });

  describe('getPayers()', function () {

    it('should have one payer for 452 and 1435', function () {

      tester.$httpBackend.expect('GET', _fixtures2.default.URI_452).respond(200, _fixtures2.default.VALID);
      avUserAuthorizations.getPayers('452', '1435').then(function (payers) {
        expect(Array.isArray(payers)).toBe(true);
        expect(payers.length).toBe(1);
        var payer = payers[0];
        expect(payer.id).toBe('8001');
        expect(payer.payerId).toBe('PAYER1');
        expect(payer.payerName).toBe('Payer One');
      });
      tester.$httpBackend.flush();
    });

    it('should no payers for 452 and 1001', function () {

      tester.$httpBackend.expect('GET', _fixtures2.default.URI_452).respond(200, _fixtures2.default.EMPTY);
      avUserAuthorizations.getPayers('452', '1001').then(function (payers) {
        expect(Array.isArray(payers)).toBe(true);
        expect(payers.length).toBe(0);
      });
      tester.$httpBackend.flush();
    });

    it('should no payers for 999', function () {

      tester.$httpBackend.expect('GET', _fixtures2.default.URI_999).respond(200, _fixtures2.default.EMPTY);
      avUserAuthorizations.getPayers('999', '1435').then(function (payers) {
        expect(Array.isArray(payers)).toBe(true);
        expect(payers.length).toBe(0);
      });
      tester.$httpBackend.flush();
    });
  });
}); /* global inject, describe, it, module, beforeEach, expect */